from abc import ABCMeta, abstractmethod
import six
from src.tank.core.tankjs import _exceptions as exceptions

from src.tank.core import tankjs


@six.add_metaclass(ABCMeta)
class AbstractRuntime(object):
    '''
    Abstract base class for runtime class.
    '''
    def exec_(self, source, cwd=None):
        '''Execute source by JavaScript runtime and return all output to stdout as a string.

        source -- JavaScript code to execute.
        cwd -- Directory where call JavaScript runtime. It may be ignored in some derived class.
        '''
        return self.compile('', cwd=cwd).exec_(source)

    def eval(self, source, cwd=None):
        '''Evaluate source in JavaScript runtime.

        source -- JavaScript code to evaluate.
        cwd -- Directory where call JavaScript runtime. It may be ignored in some derived class.
        '''
        return self.compile('', cwd=cwd).eval(source)

    def compile(self, source, cwd=None):
        '''Bulk source as a context object. The source can be used to execute another code.

        source -- JavaScript code to bulk.
        cwd -- Directory where call JavaScript runtime. It may be ignored in some derived class.
        '''
        if not self.is_available():
            raise exceptions.RuntimeUnavailableError
        return self._compile(source, cwd=cwd)

    @abstractmethod
    def is_available(self):
        raise NotImplementedError

    @abstractmethod
    def _compile(self, source, cwd=None):
        raise NotImplementedError



@six.add_metaclass(ABCMeta)
class AbstractRuntimeContext(object):
    '''
    Abstract base class for runtime context class.
    '''
    def exec_(self, source):
        '''Execute source by JavaScript runtime and return all output to stdout as a string.

        source -- JavaScript code to execute.
        '''
        if not self.is_available():
            raise tankjs.RuntimeUnavailableError
        return self._exec_(source)

    def eval(self, source):
        '''Evaluate source in JavaScript runtime.

        source -- JavaScript code to evaluate.
        '''
        if not self.is_available():
            raise tankjs.RuntimeUnavailableError
        return self._eval(source)

    def call(self, name, *args):
        '''Call a JavaScript function in context.

        name -- Name of funtion object to call
        args -- Arguments for the funtion object
        '''
        if not self.is_available():
            raise tankjs.RuntimeUnavailableError
        return self._call(name, *args)

    @abstractmethod
    def is_available(self):
        raise NotImplementedError

    @abstractmethod
    def _exec_(self, source):
        raise NotImplementedError

    @abstractmethod
    def _eval(self, source):
        raise NotImplementedError

    @abstractmethod
    def _call(self, name, *args):
        raise NotImplementedError

